@@ -113,7 +113,7 @@
   "Convert GMP integer in the buffer of a pre-allocated bignum."
   (declare (optimize (speed 3) (space 3) (safety 0))
            (type bignum-type b)
-           (type bignum-index count))
+           (type bignum-length count))
   (if (zerop count)
       0
       (the unsigned-byte (%normalize-bignum b count))))
@@ -123,7 +123,7 @@
 bignum."
   (declare (optimize (speed 3) (space 3) (safety 0))
            (type bignum-type b)
-           (type bignum-index count))
+           (type bignum-length count))
   (negate-bignum-in-place b)
   (the (integer * 0) (%normalize-bignum b count)))
 
@@ -136,12 +136,12 @@ pre-allocated bignum. The allocated bignum-length must be (1+ COUNT)."
   (declare (optimize (speed 3) (space 3) (safety 0))
            (type (alien (* unsigned-long)) z)
            (type bignum-type b)
-           (type bignum-index count))
+           (type bignum-length count))
   (dotimes (i count (%normalize-bignum b (1+ count)))
     (%bignum-set b i (deref z i))))
 
 (declaim (inline blength bassert)
-         (ftype (function (integer) (values bignum-index &optional)) blength)
+         (ftype (function (integer) (values bignum-length &optional)) blength)
          (ftype (function (integer) (values bignum &optional)) bassert))
 
 (defun blength (a)
@@ -391,7 +391,7 @@ pre-allocated bignum. The allocated bignum-length must be (1+ COUNT)."
                      ,@inits
                      ,@body
                      (let ,resinits
-                       (declare (type bignum-index ,@sizes))
+                       (declare (type bignum-length ,@sizes))
                        ,@checks
                        (let ,resallocs
                        ;; copy GMP limbs into result bignum
@@ -927,15 +927,8 @@ pre-allocated bignum. The allocated bignum-length must be (1+ COUNT)."
             (mpz-pow base power))))))
 
 ;;; installation
-(defmacro with-package-locks-ignored (&body body)
-  `(handler-bind ((sb-ext:package-lock-violation
-                    (lambda (condition)
-                      (declare (ignore condition))
-                      (invoke-restart :ignore-all))))
-     ,@body))
-
 (defun install-gmp-funs ()
-  (with-package-locks-ignored
+  (sb-ext:without-package-locks
       (macrolet ((def (destination source)
                    `(setf (fdefinition ',destination)
                           (fdefinition ',source))))
@@ -952,7 +945,7 @@ pre-allocated bignum. The allocated bignum-length must be (1+ COUNT)."
   (values))
 
 (defun uninstall-gmp-funs ()
-  (with-package-locks-ignored
+  (sb-ext:without-package-locks
       (macrolet ((def (destination source)
                    `(setf (fdefinition ',destination)
                           ,(intern (format nil "*~A-FUNCTION*" source)))))
