@@ -182,7 +182,7 @@
   (length 0 :type index)
   ;; If this layout has some kind of compiler meta-info, then this is
   ;; it. If a structure, then we store the DEFSTRUCT-DESCRIPTION here.
-  (info nil)
+  (info nil :type (or null defstruct-description))
   ;; This is true if objects of this class are never modified to
   ;; contain dynamic pointers in their slots or constant-like
   ;; substructure (and hence can be copied into read-only space by
@@ -282,8 +282,8 @@
                   (make-layout :classoid (or classoid (make-undefined-classoid name)))))))))
 
 ;;; If LAYOUT is uninitialized, initialize it with CLASSOID, LENGTH,
-;;; INHERITS, and DEPTHOID, otherwise require that it be consistent
-;;; with CLASSOID, LENGTH, INHERITS, and DEPTHOID.
+;;; INHERITS, DEPTHOID, and RAW-SLOT-METADATA. Otherwise require
+;;; that it be consistent with the existing values.
 ;;;
 ;;; UNDEFINED-CLASS values are interpreted specially as "we don't know
 ;;; anything about the class", so if LAYOUT is initialized, any
@@ -371,12 +371,10 @@
                                 context length inherits depthoid
                                 raw-slot-metadata)
   (declare (type layout old-layout) (type simple-string old-context context))
-  (let ((name (layout-proper-name old-layout)))
-    (or (let ((old-inherits (layout-inherits old-layout)))
-          (or (when (mismatch old-inherits
-                              inherits
-                              :key #'layout-proper-name)
-                (warn "change in superclasses of class ~S:~%  ~
+  (let ((name (layout-proper-name old-layout))
+        (old-inherits (layout-inherits old-layout)))
+    (or (when (mismatch old-inherits inherits :key #'layout-proper-name)
+          (warn "change in superclasses of class ~S:~%  ~
                        ~A superclasses: ~S~%  ~
                        ~A superclasses: ~S"
                       name
@@ -384,18 +382,17 @@
                       (map 'list #'layout-proper-name old-inherits)
                       context
                       (map 'list #'layout-proper-name inherits))
-                t)
-              (let ((diff (mismatch old-inherits inherits)))
-                (when diff
-                  (warn
-                   "in class ~S:~%  ~
+          t)
+        (let ((diff (mismatch old-inherits inherits)))
+          (when diff
+            (warn "in class ~S:~%  ~
                     ~@(~A~) definition of superclass ~S is incompatible with~%  ~
                     ~A definition."
                    name
                    old-context
                    (layout-proper-name (svref old-inherits diff))
                    context)
-                  t))))
+            t))
         (let ((old-length (layout-length old-layout)))
           (unless (= old-length length)
             (warn "change in instance length of class ~S:~%  ~
@@ -425,8 +422,8 @@ between the ~A definition and the ~A definition"
                 name old-context context)
           t))))
 
-;;; Require that LAYOUT data be consistent with CLASS, LENGTH,
-;;; INHERITS, and DEPTHOID.
+;;; Require that LAYOUT data be consistent with CLASSOID, LENGTH,
+;;; INHERITS, DEPTHOID, and RAW-SLOT-METADATA.
 (declaim (ftype (function
                  (layout classoid index simple-vector layout-depthoid
                          layout-raw-slot-metadata-type))
@@ -793,7 +790,6 @@ between the ~A definition and the ~A definition"
              (warn "redefining DEFTYPE type to be a class: ~
                     ~/sb-impl::print-symbol-with-prefix/" name)
              (clear-info :type :expander name)
-             (clear-info :type :lambda-list name)
              (clear-info :type :source-location name)))
 
           (remhash name table)
@@ -807,8 +803,7 @@ between the ~A definition and the ~A definition"
                        (or (info :type :kind name) :defined))
                  (let ((translation (built-in-classoid-translation new-value)))
                    (when translation
-                     (setf (info :type :translator name)
-                           (lambda (c) (declare (ignore c)) translation)))))
+                     (setf (info :type :translator name) translation))))
                 (t
                  (setf (info :type :kind name) :instance)))
           (setf (classoid-cell-classoid cell) new-value)
@@ -1460,7 +1455,7 @@ between the ~A definition and the ~A definition"
                            nil
                            (mapcar #'find-classoid direct-superclasses)))))
         (mark-ctype-interned classoid)
-        (setf (info :type :kind name) #+sb-xc-host :defined #-sb-xc-host :primitive
+        (setf (info :type :kind name) :primitive
               (classoid-cell-classoid (find-classoid-cell name :create t)) classoid)
         (unless trans-p
           (setf (info :type :builtin name) classoid))
@@ -1490,6 +1485,9 @@ between the ~A definition and the ~A definition"
 ;;; is loaded and the class defined.
 (!cold-init-forms
   (/show0 "about to define temporary STANDARD-CLASSes")
+  ;; You'd think with all the pedantic explanation in here it would at least
+  ;; be right, but it isn't: layout-inherits for FUNDAMENTAL-STREAM
+  ;; ends up as (T SLOT-OBJECT STREAM STANDARD-OBJECT)
   (dolist (x '(;; Why is STREAM duplicated in this list? Because, when
                ;; the inherits-vector of FUNDAMENTAL-STREAM is set up,
                ;; a vector containing the elements of the list below,
