@@ -1117,20 +1117,10 @@ unless :NAMED is also specified.")))
 
     (let ((predicate-name (dd-predicate-name dd)))
       (when predicate-name
-        ;; Provide inline expansion (or not).
         (when check-inlining
           (push predicate-name fnames))
-        ;; Let the predicate be inlined.
-        (setf (info :function :inline-expansion-designator predicate-name)
-              ;; The memory saved by this strange representation has got to be
-              ;; miniscule, if anything. Can we just store in the obvious way?
-              (lambda ()
-                   `(lambda (x)
-                      ;; This dead simple definition works because the
-                      ;; type system knows how to generate inline type
-                      ;; tests for instances.
-                      (typep x ',(dd-name dd))))
-              (info :function :inlinep predicate-name) :inline)))
+        (setf (info :function :source-transform predicate-name)
+              (cons dd :predicate))))
 
     (dolist (dsd (dd-slots dd))
       (let ((accessor-name (dsd-accessor-name dsd)))
@@ -1303,6 +1293,7 @@ or they must be declared locally notinline at each call site.~@:>")
 ;;; case it is the old layout.
 (defun ensure-structure-class (info inherits old-context new-context
                                     &key compiler-layout)
+  (declare (type defstruct-description info))
   (multiple-value-bind (class old-layout)
       (destructuring-bind
           (&optional
@@ -1329,27 +1320,31 @@ or they must be declared locally notinline at each call site.~@:>")
             (t
              (list (layout-classoid
                     (svref inherits (1- (length inherits))))))))
-    (let ((new-layout (make-layout :classoid class
-                                   :inherits inherits
-                                   :depthoid (length inherits)
-                                   :length (dd-layout-length info)
-                                   :info info
-                                   . #!-interleaved-raw-slots
-                                     (:n-untagged-slots (dd-raw-length info))
-                                     #!+interleaved-raw-slots
-                                     (:untagged-bitmap (dd-bitmap info))))
-          (old-layout (or compiler-layout old-layout)))
+    (let* ((old-layout (or compiler-layout old-layout))
+           (new-layout
+            (when (or (not old-layout) *type-system-initialized*)
+              (make-layout :classoid class
+                           :inherits inherits
+                           :depthoid (length inherits)
+                           :length (dd-layout-length info)
+                           :info info . #!-interleaved-raw-slots
+                                        (:n-untagged-slots (dd-raw-length info))
+                                        #!+interleaved-raw-slots
+                                        (:untagged-bitmap (dd-bitmap info))))))
       (cond
        ((not old-layout)
         (values class new-layout nil))
+       ((not new-layout)
+        ;; The assignment of INFO here can almost be deleted,
+        ;; except for a few magical types that don't d.t.r.t. in cold-init:
+        ;;  STRUCTURE-OBJECT, CONDITION, ALIEN-VALUE, INTERPRETED-FUNCTION
+        (setf (layout-info old-layout) info)
+        (values class old-layout nil))
        (;; This clause corresponds to an assertion in REDEFINE-LAYOUT-WARNING
         ;; of classic CMU CL. I moved it out to here because it was only
         ;; exercised in this code path anyway. -- WHN 19990510
         (not (eq (layout-classoid new-layout) (layout-classoid old-layout)))
         (error "shouldn't happen: weird state of OLD-LAYOUT?"))
-       ((not *type-system-initialized*)
-        (setf (layout-info old-layout) info)
-        (values class old-layout nil))
        ((redefine-layout-warning old-context
                                  old-layout
                                  new-context
@@ -1360,20 +1355,15 @@ or they must be declared locally notinline at each call site.~@:>")
         (values class new-layout old-layout))
        (t
         (let ((old-info (layout-info old-layout)))
-          (typecase old-info
-            ((or defstruct-description)
+          (if old-info
              (cond ((redefine-structure-warning class old-info info)
                     (values class new-layout old-layout))
                    (t
                     (setf (layout-info old-layout) info)
-                    (values class old-layout nil))))
-            (null
-             (setf (layout-info old-layout) info)
-             (values class old-layout nil))
-            (t
-             (error "shouldn't happen! strange thing in LAYOUT-INFO:~%  ~S"
-                    old-layout)
-             (values class new-layout old-layout)))))))))
+                    (values class old-layout nil)))
+             (progn
+               (setf (layout-info old-layout) info)
+               (values class old-layout nil)))))))))
 
 ;;; Return a list of pairs (name . index). Used for :TYPE'd
 ;;; constructors to find all the names that we have to splice in &
@@ -1990,4 +1980,10 @@ or they must be declared locally notinline at each call site.~@:>")
           (errorp
            (error "No DEFSTRUCT-DESCRIPTION for ~S." name)))))
 
+(defun structure-instance-accessor-p (name)
+  (let ((info (info :function :source-transform name)))
+    (and (listp info)
+         (defstruct-slot-description-p (cdr info))
+         info)))
+
 (/show0 "code/defstruct.lisp end of file")
