@@ -141,8 +141,16 @@
 ;;; the fop stack. But if the fop is defined to receive an argument (or two)
 ;;; then loader's main loop is responsible for supplying it.
 (defvar *fop-funs* (make-array 256 :initial-element 0))
-(declaim (type (simple-bit-vector 64) *fop-argp*))
-(defglobal *fop-argp* (make-array 64 :element-type 'bit :initial-element 0))
+
+;; Two bitmaps indicate function signature. One tells whether the fop takes
+;; operands (other than from the stack). Each consecutive block of 4 opcodes
+;; either does or doesn't, so the array is 1/4th the size of the opcode space.
+;; The other tells whether the fop wants its result pushed on the stack.
+(declaim (type (cons (simple-bit-vector 64) (simple-bit-vector 256))
+               *fop-signatures*))
+(defglobal *fop-signatures*
+    (cons (make-array 64 :element-type 'bit :initial-element 0)
+          (make-array 256 :element-type 'bit :initial-element 0)))
 
 ;;;; variables
 
