@@ -17,6 +17,14 @@
 
 (declaim (maybe-inline get get3 %put getf remprop %putf get-properties keywordp))
 
+#-sb-xc-host
+(eval-when (:compile-toplevel :load-toplevel :execute)
+  (defun handle-deprecated-global-variable (name)
+    (multiple-value-bind (state since replacements)
+        (check-deprecated-variable name)
+      (when (eq state :final)
+        `(deprecation-error ,since ',name '(,@replacements))))))
+
 (defun symbol-value (symbol)
   #!+sb-doc
   "Return SYMBOL's current bound value."
@@ -27,8 +35,8 @@
 (define-compiler-macro symbol-value (&whole form symbol &environment env)
   (when (sb!xc:constantp symbol env)
     (let ((name (constant-form-value symbol env)))
-      (when (symbolp name)
-        (check-deprecated-variable name))))
+      (awhen (and (symbolp name) (handle-deprecated-global-variable name))
+        (return-from symbol-value it))))
   form)
 
 (defun boundp (symbol)
@@ -54,6 +62,14 @@ distinct from the global value. Can also be SETF."
   (declare (optimize (safety 1)))
   (symbol-global-value symbol))
 
+#-sb-xc-host
+(define-compiler-macro symbol-global-value (&whole form symbol &environment env)
+  (when (sb!xc:constantp symbol env)
+    (let ((name (constant-form-value symbol env)))
+      (awhen (and (symbolp name) (handle-deprecated-global-variable name))
+        (return-from symbol-global-value it))))
+  form)
+
 (defun set-symbol-global-value (symbol new-value)
   (about-to-modify-symbol-value symbol 'set new-value)
   (%set-symbol-global-value symbol new-value))
@@ -116,13 +132,20 @@ distinct from the global value. Can also be SETF."
 (defun symbol-function (symbol)
   #!+sb-doc
   "Return SYMBOL's current function definition. Settable with SETF."
-  (!coerce-name-to-fun symbol-fdefn symbol))
+  (%coerce-name-to-fun symbol symbol-fdefn))
 
 (defun (setf symbol-function) (new-value symbol)
   (declare (type symbol symbol) (type function new-value))
   (with-single-package-locked-error
       (:symbol symbol "setting the symbol-function of ~A")
-    (setf (%coerce-name-to-fun symbol) new-value)))
+    ;; This code is a little "surprising" in that it is not just a limited
+    ;; case of (SETF FDEFINITION), but instead a different thing.
+    ;; I really think the code paths should be reconciled.
+    ;; e.g. what's up with *USER-HASH-TABLE-TESTS* being checked
+    ;; in %SET-FDEFINITION but not here?
+    (maybe-clobber-ftype symbol)
+    (let ((fdefn (find-or-create-fdefn symbol)))
+      (setf (fdefn-fun fdefn) new-value))))
 
 ;;; Accessors for the dual-purpose info/plist slot
 
@@ -163,7 +186,7 @@ distinct from the global value. Can also be SETF."
     ;; fops in genesis's fasload. Anonymizing the constant works around the
     ;; issue, at the expense of an extra copy of the empty info vector.
     (let ((new-vect (funcall update-fn
-                             (or current-vect #.sb!c::+nil-packed-infos+))))
+                             (or current-vect #.+nil-packed-infos+))))
       (unless (simple-vector-p new-vect)
         (aver (null new-vect))
         (return)) ; nothing to do
