@@ -244,7 +244,7 @@ maintained."
                                  (prog1 (aref ,f-buffer ,f-index)
                                    (incf ,f-index))))))))
          (declare (inline fast-read-byte))
-         (declare (enable-package-locks read-byte))
+         (declare (enable-package-locks fast-read-byte))
          (unwind-protect
               (locally ,@body)
            (setf (ansi-stream-in-index ,f-stream) ,f-index))))))
@@ -259,7 +259,7 @@ maintained."
     `(let ((,index ,start))
        (loop
         (cond ((< (truly-the index ,index) (length ,rest-var))
-               (let ((,var (sb!c::fast-&rest-nth ,index ,rest-var))
+               (let ((,var (fast-&rest-nth ,index ,rest-var))
                      ,@(if index-var `((,index-var ,index))))
                  ,@body)
                (incf ,index))
