@@ -1411,7 +1411,7 @@ session."
          ;; internal printer variables
          (sb!impl::*previous-case* nil)
          (sb!impl::*previous-readtable-case* nil)
-         (sb!impl::*internal-symbol-output-fun* nil)
+         (sb!impl::*internal-symbol-output-fun* #'error)
          (sb!impl::*descriptor-handlers* nil)) ; serve-event
     (declare (inline make-restart)) ;; to allow DX-allocation
     ;; Binding from C
