@@ -52,6 +52,16 @@
   (declare (type ir2-block 2block))
   (block-physenv (ir2-block-block 2block)))
 
+(defun leaf-visible-to-debugger-p (leaf node)
+  (or (rassoc leaf (lexenv-vars (node-lexenv node)))
+      (labels ((visible-in-call-lexenv (lambda)
+                 (when lambda
+                   (let ((call-lexenv (lambda-call-lexenv lambda)))
+                     (and call-lexenv
+                          (or (rassoc leaf (lexenv-vars call-lexenv))
+                              (visible-in-call-lexenv (lexenv-lambda call-lexenv))))))))
+        (visible-in-call-lexenv (lexenv-lambda (node-lexenv node))))))
+
 ;;; Given a local conflicts vector and an IR2 block to represent the
 ;;; set of live TNs, and the VAR-LOCS hash-table representing the
 ;;; variables dumped, compute a bit-vector representing the set of
@@ -72,7 +82,7 @@
         (when (and (lambda-var-p leaf)
                    (or (not (member (tn-kind tn)
                                     '(:environment :debug-environment)))
-                       (rassoc leaf (lexenv-vars (node-lexenv node))))
+                       (leaf-visible-to-debugger-p leaf node))
                    (or (null spilled)
                        (not (member tn spilled))))
           (let ((num (gethash leaf var-locs)))
@@ -97,9 +107,7 @@
 
   (let ((byte-buffer *byte-buffer*))
     (vector-push-extend
-     (dpb (position-or-lose kind *compiled-code-location-kinds*)
-          compiled-code-location-kind-byte
-          0)
+     (position-or-lose kind *compiled-code-location-kinds*)
      byte-buffer)
 
     (let ((loc (if (fixnump label) label (label-position label))))
@@ -184,70 +192,39 @@
     (dump-location-from-info loc tlf-num var-locs))
   (values))
 
-;;; Dump the successors of Block, being careful not to fly into space
-;;; on weird successors.
-(defun dump-block-successors (block physenv)
-  (declare (type cblock block) (type physenv physenv))
-  (let* ((tail (component-tail (block-component block)))
-         (succ (block-succ block))
-         (valid-succ
-          (if (and succ
-                   (or (eq (car succ) tail)
-                       (not (eq (block-physenv (car succ)) physenv))))
-              ()
-              succ))
-         (byte-buffer *byte-buffer*))
-    (vector-push-extend
-     (dpb (length valid-succ) compiled-debug-block-nsucc-byte 0)
-     byte-buffer)
-    (let ((base (block-number
-                 (node-block
-                  (lambda-bind (physenv-lambda physenv))))))
-      (dolist (b valid-succ)
-        (write-var-integer
-         (the index (- (block-number b) base))
-         byte-buffer))))
-  (values))
-
 ;;; Return a vector and an integer (or null) suitable for use as the
-;;; BLOCKS and TLF-NUMBER in FUN's DEBUG-FUN. This requires two
-;;; passes to compute:
-;;; -- Scan all blocks, dumping the header and successors followed
-;;;    by all the non-elsewhere locations.
-;;; -- Dump the elsewhere block header and all the elsewhere
-;;;    locations (if any.)
+;;; BLOCKS and TLF-NUMBER in FUN's DEBUG-FUN.
 (defun compute-debug-blocks (fun var-locs)
   (declare (type clambda fun) (type hash-table var-locs))
   (let ((*previous-location* 0)
         (tlf-num (find-tlf-number fun))
         (physenv (lambda-physenv fun))
-        (prev-locs nil)
-        (prev-block nil)
-        (byte-buffer *byte-buffer*))
+        (byte-buffer *byte-buffer*)
+        prev-block
+        locations
+        elsewhere-locations)
     (setf (fill-pointer byte-buffer) 0)
-    (collect ((elsewhere))
-      (do-physenv-ir2-blocks (2block physenv)
-        (let ((block (ir2-block-block 2block)))
-          (when (eq (block-info block) 2block)
-            (when prev-block
-              (dump-block-locations prev-block prev-locs tlf-num var-locs))
-            (setq prev-block block  prev-locs ())
-            (dump-block-successors block physenv)))
+    (do-physenv-ir2-blocks (2block physenv)
+      (let ((block (ir2-block-block 2block)))
+        (when (eq (block-info block) 2block)
+          (when prev-block
+            (dump-block-locations prev-block (nreverse (shiftf locations nil))
+                                  tlf-num var-locs))
+          (setf prev-block block)))
+      (dolist (loc (ir2-block-locations 2block))
+        (if (label-elsewhere-p (location-info-label loc)
+                               (location-info-kind loc))
+            (push loc elsewhere-locations)
+            (push loc locations))))
 
-        (collect ((here prev-locs))
-          (dolist (loc (ir2-block-locations 2block))
-            (if (label-elsewhere-p (location-info-label loc))
-                (elsewhere loc)
-                (here loc)))
-          (setq prev-locs (here))))
+    (dump-block-locations prev-block (nreverse locations)
+                          tlf-num var-locs)
 
-      (dump-block-locations prev-block prev-locs tlf-num var-locs)
-
-      (when (elsewhere)
-        (vector-push-extend compiled-debug-block-elsewhere-p byte-buffer)
-        (write-var-integer (length (elsewhere)) byte-buffer)
-        (dolist (loc (elsewhere))
-          (dump-location-from-info loc tlf-num var-locs))))
+    (when elsewhere-locations
+      (write-var-integer (length elsewhere-locations) byte-buffer)
+      (dolist (loc (nreverse elsewhere-locations))
+        (push loc locations)
+        (dump-location-from-info loc tlf-num var-locs)))
 
     (values (!make-specialized-array (length byte-buffer) '(unsigned-byte 8)
                                      byte-buffer)
@@ -296,11 +273,11 @@
             (frob i))
           (dovector (i seq (setq length (length seq)))
             (frob i)))
-      (let ((specializer `(unsigned-byte
-                           ,(etypecase maxoid
-                              ((unsigned-byte 8) 8)
-                              ((unsigned-byte 16) 16)
-                              ((unsigned-byte 32) 32)))))
+      (let ((specializer (etypecase maxoid
+                           ((unsigned-byte 8) '(unsigned-byte 8))
+                           ((unsigned-byte 16) '(unsigned-byte 16))
+                           ((unsigned-byte 32) '(unsigned-byte 32))
+                           ((unsigned-byte 64) '(unsigned-byte 64)))))
         ;; formerly (coerce seq `(simple-array ,specializer (*)))
         ;; plus a kludge for cross-compilation. This is nicer.
         (!make-specialized-array length specializer seq)))))
@@ -339,7 +316,11 @@
          (save-tn (and tn (tn-save-tn tn)))
          (kind (and tn (tn-kind tn)))
          (flags 0)
-         (info (lambda-var-arg-info var)))
+         (info (lambda-var-arg-info var))
+         (indirect (and (lambda-var-indirect var)
+                        (not (lambda-var-explicit-value-cell var))
+                        (neq (lambda-physenv fun)
+                             (lambda-physenv (lambda-var-home var))))))
     (declare (type index flags))
     (when minimal
       (setq flags (logior flags compiled-debug-var-minimal-p))
@@ -356,6 +337,8 @@
       (setq flags (logior flags compiled-debug-var-save-loc-p)))
     (unless (or (zerop id) minimal)
       (setq flags (logior flags compiled-debug-var-id-p)))
+    (when indirect
+      (setq flags (logior flags compiled-debug-var-indirect-p)))
     (when info
       (case (arg-info-kind info)
         (:more-context
@@ -367,11 +350,21 @@
       (vector-push-extend name buffer)
       (unless (zerop id)
         (vector-push-extend id buffer)))
-    (if (and tn (tn-offset tn))
-        (vector-push-extend (tn-sc-offset tn) buffer)
-        (aver minimal))
-    (when save-tn
-      (vector-push-extend (tn-sc-offset save-tn) buffer)))
+    (cond (indirect
+           ;; Indirect variables live in the parent frame, and are
+           ;; accessed through a saved frame pointer.
+           ;; The first one/two sc-offsets are for the frame pointer,
+           ;; the third is for the stack offset.
+           (vector-push-extend (tn-sc-offset tn) buffer)
+           (when save-tn
+             (vector-push-extend (tn-sc-offset save-tn) buffer))
+           (vector-push-extend (tn-sc-offset (leaf-info var)) buffer))
+          (t
+           (if (and tn (tn-offset tn))
+               (vector-push-extend (tn-sc-offset tn) buffer)
+               (aver minimal))
+           (when save-tn
+             (vector-push-extend (tn-sc-offset save-tn) buffer)))))
   (values))
 
 ;;; Return a vector suitable for use as the DEBUG-FUN-VARS
