@@ -1588,7 +1588,7 @@
 
 ;; FIXME: This function does not return, but due to the implementation
 ;; of FILTER-LVAR we cannot write it here.
-(defknown %compile-time-type-error (t t t t) *)
+(defknown %compile-time-type-error (t t t t t) *)
 (defknown case-failure (t t t) nil)
 
 (defknown %odd-key-args-error () nil)
@@ -1627,7 +1627,6 @@
   (values (simple-array * (*)) index index index)
   (foldable flushable))
 (defknown %set-symbol-package (symbol t) t ())
-(defknown %coerce-name-to-fun ((or symbol cons)) function (flushable))
 (defknown %coerce-callable-to-fun (callable) function
     (flushable explicit-check))
 (defknown array-bounding-indices-bad-error (t t t) nil)
