@@ -73,20 +73,6 @@
   (when (boundp '*free-funs*)       ; when compiling
     (remhash name *free-funs*))
 
-  ;; It's uncommon for users to define both a SETF macro and function for
-  ;; the same thing, so usually we warn about it, but offer a somewhat arbitrary
-  ;; escape to avoid a warning - if the function gets an ftype proclaimed
-  ;; before DEFSETF (or -EXPANDER) is used, we don't complain about the macro.
-  ;; SBCL itself provides most SETFable things both ways even when CLHS does not
-  ;; say specifically that #'(SETF x) exists.
-  ;; But when building SBCL the order of function/macro definition is not
-  ;; directly controlled because most SETF functions are defined en masse by
-  ;; 'setf-funs' with no regard for what exists, and we also have to respect
-  ;; all other constraints on build order to make bootstrap work.
-  ;; So, ... just don't warn when running the cross-compiler, which means
-  ;; don't compile this line in when building the cross-compiler.
-  #-sb-xc-host(note-if-setf-fun-and-macro name)
-
   (values))
 
 ;;; This is called to do something about SETF functions that overlap
@@ -95,14 +81,17 @@
 ;;; warning. Due to the weak semantics of the (SETF FUNCTION) name, we
 ;;; can't assume that they aren't just naming a function (SETF FOO)
 ;;; for the heck of it. NAME is already known to be well-formed.
-(defun note-if-setf-fun-and-macro (name)
-  (when (and (consp name)
-             (eq (car name) 'setf))
-    (let ((stem (second name)))
-      (when (or (info :setf :inverse stem) (info :setf :expander stem))
-        (compiler-style-warn
+(defun warn-if-setf-macro (name)
+  ;; Never warn about this situation when running the cross-compiler.
+  ;; SBCL provides expanders/inverses *and* functions for most SETFable things
+  ;; even when CLHS does not specifically state that #'(SETF x) exists.
+  #+sb-xc-host (declare (ignore name))
+  #-sb-xc-host
+  (let ((stem (second name)))
+    (when (or (info :setf :inverse stem) (info :setf :expander stem))
+      (compiler-style-warn
          "defining function ~S when ~S already has a SETF macro"
-         name stem))))
+         name stem)))
   (values))
 
 ;;; Make NAME no longer be a function name: clear everything back to
@@ -112,8 +101,8 @@
     (macrolet ((frob (&rest types)
                  `(clear-info-values
                    name ',(mapcar (lambda (x)
-                                    (type-info-number
-                                     (type-info-or-lose :function x)))
+                                    (meta-info-number
+                                     (meta-info-or-lose :function x)))
                                   types))))
       ;; Note that this does not clear the :DEFINITION.
       ;; That's correct, because if we lose the association between a
@@ -139,14 +128,10 @@
     (if (info :function :assumed-type name)
         (clear-info :function :assumed-type name))))
 
-;;; Decode any raw (INFO :FUNCTION :INLINE-EXPANSION-DESIGNATOR FUN-NAME)
-;;; value into a lambda expression, or return NIL if there is none.
+;;; Trivially wrap (INFO :FUNCTION :INLINE-EXPANSION-DESIGNATOR FUN-NAME)
 (declaim (ftype (function ((or symbol cons)) list) fun-name-inline-expansion))
 (defun fun-name-inline-expansion (fun-name)
-  (let ((info (info :function :inline-expansion-designator fun-name)))
-    (if (functionp info)
-        (funcall info)
-        info)))
+  (info :function :inline-expansion-designator fun-name))
 
 ;;;; ANSI Common Lisp functions which are defined in terms of the info
 ;;;; database
@@ -287,7 +272,7 @@ return NIL. Can be set with SETF when ENV is NIL."
   (declare (type (or null string) string))
   (let ((info-number
          (macrolet ((info-number (class type)
-                      (type-info-number (type-info-or-lose class type))))
+                      (meta-info-number (meta-info-or-lose class type))))
            (case doc-type
              (variable (info-number :variable :documentation))
              (structure
