@@ -691,6 +691,11 @@ necessary, since type inference may take arbitrarily long to converge.")
         (print-loop (component-outer-loop component))))
     |#
 
+    ;; This should happen at some point before PHYSENV-ANALYZE, and
+    ;; after RECORD-COMPONENT-XREFS.  Beyond that, I haven't really
+    ;; thought things through.  -- AJB, 2014-Jun-08
+    (eliminate-dead-code component)
+
     ;; FIXME: What is MAYBE-MUMBLE for? Do we need it any more?
     (maybe-mumble "env ")
     (physenv-analyze component)
@@ -1461,6 +1466,18 @@ necessary, since type inference may take arbitrarily long to converge.")
 ;;; Compile FORM and arrange for it to be called at load-time. Return
 ;;; the dumper handle and our best guess at the type of the object.
 (defun compile-load-time-value (form)
+  ;; Special case for the cross-compiler. While the normal ltv stuff is fine
+  ;; for the most part, it is inadequate for SETUP-PRINTER-STATE.
+  ;; In cold-init we want the printer to work before regular ltv forms are run,
+  ;; so this is fop-based magic, slightly dangerous in that it can produce a
+  ;; use of #'F before the referenced function has been defined.
+  ;; Just be careful not to do that.
+  #+sb-xc-host
+  (when (typep form '(cons (eql function) (cons symbol null)))
+    (fopcompile form nil t)
+    (return-from compile-load-time-value
+      (values (sb!fasl::dump-pop *compile-object*)
+              (specifier-type 'function))))
   (let ((lambda (compile-load-time-stuff form t)))
     (values
      (fasl-dump-load-time-value-lambda lambda *compile-object*)
