@@ -60,7 +60,8 @@
 (deftransform typep ((object type &optional env) * * :node node)
   (unless (constant-lvar-p type)
     (give-up-ir1-transform "can't open-code test of non-constant type"))
-  (unless (and (constant-lvar-p env) (null (lvar-value env)))
+  (unless (or (null env)
+              (and (constant-lvar-p env) (null (lvar-value env))))
     (give-up-ir1-transform "environment argument present and not null"))
   (multiple-value-bind (expansion fail-p)
       (source-transform-typep 'object (lvar-value type))
@@ -896,7 +897,11 @@
            ((csubtypep tspec (specifier-type 'function))
             (if (csubtypep (lvar-type x) (specifier-type 'symbol))
                 `(coerce-symbol-to-fun x)
-                `(coerce-to-fun x)))
+                ;; if X can later be derived as FUNCTION then we don't want
+                ;; to call COERCE-TO-FUN, because there's no smartness
+                ;; that can undo that and see that it's really (IDENTITY X).
+                (progn (delay-ir1-transform node :constraint)
+                       `(coerce-to-fun x))))
            (t
             (give-up-ir1-transform
              "~@<open coding coercion to ~S not implemented.~:@>"
