@@ -53,7 +53,7 @@
 ;;;;
 ;;;; * Since the cache is used for memoization only we don't need to
 ;;;;   worry about which of simultaneous replacements (when expanding
-;;;;   the cache) takes place: the loosing one will have its work
+;;;;   the cache) takes place: the losing one will have its work
 ;;;;   redone later. This also allows us to drop entries when the
 ;;;;   cache is about to grow insanely huge.
 ;;;;
@@ -104,21 +104,14 @@
   ;; bits at the low end.
   (logand (1- vector-length) (- line-size)))
 
-;;; The smallest power of two that is equal to or greater then X.
-(declaim (inline power-of-two-ceiling))
-(defun power-of-two-ceiling (x)
-  (ash 1 (integer-length (1- x))))
-
 (defun cache-statistics (cache)
   (let* ((vector (cache-vector cache))
          (size (length vector))
          (line-size (cache-line-size cache))
-         (total-lines (/ size line-size))
-         (free-lines (loop for i from 0 by line-size below size
-                           unless (eq (svref vector i) '..empty..)
-                           count t)))
-    (values (- total-lines free-lines) total-lines
-            (cache-depth cache) (cache-limit cache))))
+         (total-lines (/ size line-size)))
+    (values (loop for i from 0 by line-size below size
+                  count (neq (svref vector i) '..empty..))
+            total-lines (cache-depth cache) (cache-limit cache))))
 
 ;;; Don't allocate insanely huge caches: this is 4096 lines for a
 ;;; value cache with 8-15 keys -- probably "big enough for anyone",
@@ -191,40 +184,53 @@
 ;;; number of keys and presence of values in the cache is known
 ;;; beforehand.
 (defun emit-cache-lookup (cache-var layout-vars miss-tag value-var)
-  (let ((line-size (power-of-two-ceiling (+ (length layout-vars)
-                                            (if value-var 1 0)))))
-    (with-unique-names (n-index n-vector n-depth n-pointer n-mask
-                       MATCH-WRAPPERS EXIT-WITH-HIT)
-      `(let* ((,n-index (hash-layout-or ,(car layout-vars) (go ,miss-tag)))
-              (,n-vector (cache-vector ,cache-var))
-              (,n-mask (cache-mask ,cache-var)))
-         (declare (index ,n-index))
+  (with-unique-names (probe n-vector n-depth n-mask
+                      MATCH-WRAPPERS EXIT-WITH-HIT)
+    (let* ((num-keys (length layout-vars))
+           (pointer
+            ;; We don't need POINTER if the cache has 1 key and no value,
+            ;; or if FOLD-INDEX-ADDRESSING is supported, in which case adding
+            ;; a constant to the base index for each cell-ref yields better code.
+            #-(or x86 x86-64)
+            (when (or (> num-keys 1) value-var) (make-symbol "PTR")))
+           (line-size (power-of-two-ceiling (+ num-keys (if value-var 1 0)))))
+      `(let ((,n-mask (cache-mask ,cache-var))
+             (,probe (hash-layout-or ,(car layout-vars) (go ,miss-tag))))
+         (declare (index ,probe))
          ,@(mapcar (lambda (layout-var)
-                     `(mixf ,n-index (hash-layout-or ,layout-var (go ,miss-tag))))
+                     `(mixf ,probe (hash-layout-or ,layout-var (go ,miss-tag))))
                    (cdr layout-vars))
          ;; align with cache lines
-         (setf ,n-index (logand ,n-index ,n-mask))
+         (setf ,probe (logand ,probe ,n-mask))
          (let ((,n-depth (cache-depth ,cache-var))
-               (,n-pointer ,n-index))
-           (declare (index ,n-depth ,n-pointer))
+               (,n-vector (cache-vector ,cache-var))
+               ,@(when pointer `((,pointer ,probe))))
+           (declare (index ,n-depth ,@(when pointer (list pointer))))
            (tagbody
             ,MATCH-WRAPPERS
-              (when (and ,@(mapcar
-                            (lambda (layout-var)
-                              `(prog1
-                                   (eq ,layout-var (svref ,n-vector ,n-pointer))
-                                 (incf ,n-pointer)))
-                            layout-vars))
-                ,@(when value-var
-                    `((setf ,value-var (non-empty-or (svref ,n-vector ,n-pointer)
-                                                     (go ,miss-tag)))))
-                (go ,EXIT-WITH-HIT))
-              (if (zerop ,n-depth)
-                  (go ,miss-tag)
-                  (decf ,n-depth))
-              (setf ,n-index (next-cache-index ,n-mask ,n-index ,line-size)
-                    ,n-pointer ,n-index)
-              (go ,MATCH-WRAPPERS)
+            (when (and ,@(loop for layout-var in layout-vars
+                               for i from 0
+                               collect
+                             (if pointer
+                                 `(prog1 (eq ,layout-var
+                                             (svref ,n-vector ,pointer))
+                                    (incf ,pointer))
+                                 `(eq ,layout-var
+                                      (svref ,n-vector
+                                             (the index (+ ,probe ,i)))))))
+              ,@(when value-var
+                 `((setf ,value-var
+                         (non-empty-or (svref ,n-vector
+                                              ,(or pointer
+                                                   `(the index
+                                                         (+ ,probe ,num-keys))))
+                                       (go ,miss-tag)))))
+              (go ,EXIT-WITH-HIT))
+            (when (zerop ,n-depth) (go ,miss-tag))
+            (decf ,n-depth)
+            (setf ,probe (next-cache-index ,n-mask ,probe ,line-size))
+            ,@(if pointer `((setf ,pointer ,probe)))
+            (go ,MATCH-WRAPPERS)
             ,EXIT-WITH-HIT))))))
 
 ;;; Probes CACHE for LAYOUTS.
@@ -326,7 +332,8 @@
 ;;;; incomplete lines.
 (defun copy-and-expand-cache (cache layouts value)
   (let ((copy (%copy-cache cache))
-        (length (length (cache-vector cache))))
+        (length (length (cache-vector cache)))
+        (drop-random-entries nil))
     (declare (index length))
     (when (< length +cache-vector-max-length+)
       (setf length (* 2 length)))
@@ -340,23 +347,35 @@
              (cache-mask copy) (compute-cache-mask length (cache-line-size cache))
              (cache-limit copy) (compute-limit (/ length (cache-line-size cache))))
        ;; First insert the new one -- if we don't do this first and
-       ;; the cache has reached it's maximum size we may end up
-       ;; looping in FILL-CACHE.
+       ;; the cache has reached its maximum size we may end up looping
+       ;; in FILL-CACHE.
        (unless (try-update-cache copy layouts value)
-         (bug "Could not insert ~S:~S to supposedly empty ~S." layouts value cache))
-       (map-cache (lambda (layouts value)
-                    (unless (try-update-cache copy layouts value)
-                      ;; If the cache would grow too much we drop the
-                      ;; remaining the entries that don't fit. FIXME:
-                      ;; It would be better to drop random entries to
-                      ;; avoid getting into a rut here (best done by
-                      ;; making MAP-CACHE map in a random order?), and
-                      ;; possibly to downsize the cache more
-                      ;; aggressively (on the assumption that most
-                      ;; entries aren't getting used at the moment.)
-                      (when (< length +cache-vector-max-length+)
-                        (setf length (* 2 length))
-                        (go :again))))
+         (bug "Could not insert ~S:~S to supposedly empty ~S." layouts value copy))
+       (map-cache (if drop-random-entries
+                      ;; The cache is at maximum size, and all entries
+                      ;; do not fit in. Drop a random ~50% of entries,
+                      ;; to make space for new ones. This needs to be
+                      ;; random, since otherwise we might get in a
+                      ;; rut: add A causing B to drop, then add B
+                      ;; causing A to drop... repeat ad nauseam,
+                      ;; spending most of the time here instead of
+                      ;; doing real work. 50% because if we drop too
+                      ;; few we need to do this almost right away
+                      ;; again, and if we drop too many, we need to
+                      ;; recompute more then we'd like.
+                      ;; _Experimentally_ 50% seems to perform the
+                      ;; best, but it would be nice to have a proper
+                      ;; analysis...
+                      (randomly-punting-lambda (layouts value)
+                        (try-update-cache copy layouts value))
+                      (lambda (layouts value)
+                        (unless (try-update-cache copy layouts value)
+                          ;; Didn't fit -- expand the cache, or drop
+                          ;; a few unlucky ones.
+                          (if (< length +cache-vector-max-length+)
+                              (setf length (* 2 length))
+                              (setf drop-random-entries t))
+                          (go :again))))
                   cache))
     copy))
 
@@ -398,18 +417,18 @@
 ;;; necessary, and returns the new cache.
 (defun fill-cache (cache layouts value)
   (labels
-      ((%fill-cache (cache layouts value)
+      ((%fill-cache (cache layouts value expand)
          (cond ((try-update-cache cache layouts value)
                 cache)
-               ((cache-has-invalid-entries-p cache)
+               ((and (not expand) (cache-has-invalid-entries-p cache))
                 ;; Don't expand yet: maybe there will be enough space if
                 ;; we just drop the invalid entries.
-                (%fill-cache (copy-cache cache) layouts value))
+                (%fill-cache (copy-cache cache) layouts value t))
                (t
                 (copy-and-expand-cache cache layouts value)))))
     (if (listp layouts)
-        (%fill-cache cache layouts value)
-        (%fill-cache cache (list layouts) value))))
+        (%fill-cache cache layouts value nil)
+        (%fill-cache cache (list layouts) value nil))))
 
 ;;; Calls FUNCTION with all layouts and values in cache.
 (defun map-cache (function cache)
