@@ -49,15 +49,27 @@
 #include "interrupt.h"
 #include "lispregs.h"
 
-#if defined(LISP_FEATURE_WIN32) && defined(LISP_FEATURE_SB_THREAD)
+#ifdef LISP_FEATURE_SB_THREAD
+
+#ifdef LISP_FEATURE_OPENBSD
+#include <pthread_np.h>
+#endif
+
+#ifdef LISP_FEATURE_SUNOS
+#include <thread.h>
+#endif
+
+#ifdef LISP_FEATURE_WIN32
 # define IMMEDIATE_POST_MORTEM
 #endif
 
-#if defined(LISP_FEATURE_DARWIN) && defined(LISP_FEATURE_SB_THREAD)
+#ifdef LISP_FEATURE_DARWIN
 #define DELAY_THREAD_POST_MORTEM 5
 #define LOCK_CREATE_THREAD
 #endif
 
+#endif
+
 #if defined(LISP_FEATURE_FREEBSD) || defined(LISP_FEATURE_DRAGONFLY)
 #define LOCK_CREATE_THREAD
 #endif
@@ -516,13 +528,27 @@ attach_os_thread(init_thread_data *scribble)
      * stack. */
     void *stack_addr;
     size_t stack_size;
-#ifdef LISP_FEATURE_DARWIN
+#ifdef LISP_FEATURE_OPENBSD
+    stack_t stack;
+    pthread_stackseg_np(os, &stack);
+    stack_size = stack.ss_size;
+    stack_addr = (void*)((size_t)stack.ss_sp - stack_size);
+#elif defined LISP_FEATURE_SUNOS
+  stack_t stack;
+  thr_stksegment(&stack);
+  stack_size = stack.ss_size;
+  stack_addr = (void*)((size_t)stack.ss_sp - stack_size);
+#elif defined(LISP_FEATURE_DARWIN)
     stack_addr = pthread_get_stackaddr_np(os);
     stack_size = pthread_get_stacksize_np(os);
 #else
     pthread_attr_t attr;
+#ifdef LISP_FEATURE_FREEBSD
+    pthread_attr_get_np(os, &attr);
+#else
     int pthread_getattr_np(pthread_t, pthread_attr_t *);
     pthread_getattr_np(os, &attr);
+#endif
     pthread_attr_getstack(&attr, &stack_addr, &stack_size);
 #endif
 
