@@ -13,6 +13,19 @@
 
 (in-package :cl-user)
 
+;;;; Assert that entries inserted into a dispatch table with equal priority
+;;;; are order-preserving - unless they are of the form (CONS (EQL x)).
+;;;; This is not a requirement in general, but is quite reasonable.
+(with-test (:name :pprint-dispatch-order-preserving)
+  (let ((tbl (sb-pretty::make-pprint-dispatch-table)))
+    (handler-bind ((style-warning #'muffle-warning)) ; nonexistent types
+      (set-pprint-dispatch 'foo1 #'pprint-fill 5 tbl)
+      (set-pprint-dispatch 'fool #'pprint-fill 0 tbl)
+      (set-pprint-dispatch 'foo2 #'pprint-fill 5 tbl))
+    (let ((entries (sb-pretty::pprint-dispatch-table-entries tbl)))
+      (assert (equal (mapcar #'sb-pretty::pprint-dispatch-entry-type entries)
+                     '(foo1 foo2 fool))))))
+
 ;;;; tests for former BUG 99, where pretty-printing was pretty messed
 ;;;; up, e.g. PPRINT-LOGICAL-BLOCK - because of CHECK-FOR-CIRCULARITY
 ;;;; - didn't really work:
@@ -312,5 +325,14 @@
 (with-test (:name :literal-fun-nested-lists)
   (assert (search "EQUALP" (format nil "~:w" `((((((,#'equalp)))))))
                   :test #'char-equal)))
+
+(defvar *a* (make-array 3 :fill-pointer 0))
+(with-test (:name :pprint-logical-block-eval-order)
+  (flet ((vp (x) (vector-push x *a*)))
+    (pprint-logical-block (nil (progn (vp 1) '(foo))
+                               :suffix (progn (vp 2) "}")
+                               :prefix (progn (vp 3) "{"))
+      (write (pprint-pop))))
+  (assert (equalp *a* #(1 2 3))))
 
 ;;; success
